
/**************************************************************************************************
* NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
*
* Copyright 2010 Flexera Software, Inc. and/or InstallShield Co. Inc.  All Rights Reserved.  
*
* This computer program is the property of Flexera Software, Inc. and InstallShield Co. Inc. of 
* Schaumburg, Illinois, U.S.A.  Any use, copy, publication, distribution, display, modification, 
* or transmission of this computer program in whole or in part in any form or by any means without
* the prior express written permission of Flexera Software, Inc. or InstallShield Co. Inc. is 
* strictly prohibited.
* 
* Except when expressly provided by Flexera Software, Inc. or InstallShield Co. Inc. in writing,
* possession of this computer program shall not be construed to confer any license or rights under 
* any of Flexera Software, Inc.s or InstallShield Co. Inc.s intellectual property rights, whether
* by estoppel, implication, or otherwise.  
* 
* ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
**************************************************************************************************/
/*
	This sample utility demonstrates Composite Transactions.

	This module implements the "view fulfillment records" usage.

		comptranutil -view 

	Viewing fullfillments is supported by the common API and is not specific to composite 
	transactions; the fulfillments may also have been created by local activations, single action
	trnsactiosn or short code transactions.

	The common API uses two object 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "FlxActCommon.h"
#include "FlxActError.h"

#include "compTranUtilView.h"

typedef struct flxCtuViewFilter_struct
{
	FlxActBool	bViewAllIds;
	FlxActBool	bViewAllProducts;
	FlxActBool	bViewAllFeatures;
	FlxActBool	bViewTrusted;
	FlxActBool	bViewUntrusted;
	FlxActBool	bViewEnabled;
	FlxActBool	bViewDisabled;
	const char*	pszFidFilter;
	const char*	pszPidFilter;
	const char*	pszFeatureFilter;
} FlxCtuViewFilter;

typedef enum flxCtuViewType_enum
{
	flxCtuViewTypeShort	= 0,
	flxCtuViewTypeLong,
} FlxCtuViewType;


static const char* sGetFulfillmentTypeString(FlxActProdLicSpc prodSpec);
static const char* sGetFulfillmentTrustFlagsString(FlxActProdLicSpc prodSpec);
static uint32_t sGetNonOverdraftCountTotal(FlxActProdLicSpc prodSpec);
static FlxActBool sIsViewCounts(const flxCtuContext* pContext, const flxCtuCommand* pCommand);
static FlxActBool sIsViewFilterPass(FlxActProdLicSpc prodSpec, const FlxCtuViewFilter* pFilter);
static FlxActBool sHasThisFeatureToken(FlxActProdLicSpc prodSpec, const char* pszFeatureName);
static FlxActBool sHasThisFulfillmentId(FlxActProdLicSpc prodSpec, const char* pszFulfillmentId);
static FlxActBool sHasThisProductId(FlxActProdLicSpc prodSpec, const char* pszProductId);
static FlxCtuViewType sGetViewType(const flxCtuCommand* pCommand);
static void sSetViewFilter(const flxCtuCommand* pCommand, FlxCtuViewFilter* pFilter);
static void sViewFulfillmentShort(FlxActProdLicSpc prodSpec, FlxActBool bPrintHeading);
static void sViewFulfillmentLong(FlxActProdLicSpc prodSpec, FlxActBool bViewCounts);
static void sViewFulfillmentCounts(FlxActProdLicSpc prodSpec);
static void sViewNameAndFeatures(const char* pszName, FlxActProdLicSpc prodSpec);
static void sViewNameAndValue(const char* pszName, const char* pszValue);
static void sViewNameAndUint32(const char* pszName, uint32_t value);

/***************************************************************************************************
*	flxCtuDeleteShowUsage
*
* Show the command line usage for the -delete command.
***************************************************************************************************/
void flxCtuDeleteShowUsage(flxCtuContext* pContext)
{
	flxCtuPrintOutput(
		"\n"
		"deleteFR command: delete fulfillment record(s) from trusted storage.\n"
		"========\n"
		"    -deleteFR [<fulfillmentid>] [confirm=yes|no] [<filters>]\n"	
		"\n"
		"  Filters:\n"
		"    fulfillmentId        The record with this id.\n"
		"    product=<productId>  Only records with this ProductId.\n"
		"    feature=<string>     Only records with <string> in the feature line.\n"
		"    trusted=yes|no       Only trusted|untrusted records.\n"
		"    enabled=yes|no       Only enabled|disabled records.\n"
		"\n"
		"  The fulfillments to be deleted are displayed and confirmation requested\n"
		"  (use confirm=no to suppress confirmation).\n"
		"\n"
		"  Example - delete single fulfillment record\n"
		"      -delete MyFulfillmentId\n"
		"\n"
		"  Example - delete fulfillment records for \"MyProduct\"\n"
		"      -delete product=MyProduct\n"
		"\n"
		"  Example - delete untrusted fulfillment recordss\n"
		"      -delete trusted=no\n"
		);
}

/***************************************************************************************************
*	flxCtuDeleteFulfillments
*
* Process a delete command - delete fulfillment records.
* For command line usage see above.
***************************************************************************************************/
FlxActBool flxCtuDeleteFulfillments(flxCtuContext* pContext, const flxCtuCommand* pCommand)
{
	FlxActBool			bResult			= FLX_ACT_FALSE;
	FlxActLicSpc		licSpec			= 0;
	FlxActProdLicSpc	prodSpec		= 0;
	uint32_t			frCount			= 0;
	uint32_t			frIndex			= 0;
	uint32_t			deleteCount		= 0;
	FlxCtuViewFilter	filter;

	/*
		Set the filtering choices from the command value and attributes.
	*/
	sSetViewFilter(pCommand, &filter);

	/*
		Get the fulfillment records.
		The FlxActProdSpc object prodSpec holds details of one fulfillment record.
		The FlxActProdLicSpc object licSpec is a container of prodSpecs.
	*/
	if ( !flxActCommonLicSpcCreate(pContext->hApi, &licSpec) )
	{
		flxCtuContextPrintLastApiFlexError(pContext);
		return FLX_ACT_FALSE;
	}
	if ( !flxActCommonLicSpcPopulateAllFromTS(licSpec) )
	{
		flxCtuContextPrintLastApiFlexError(pContext);
		flxActCommonLicSpcDelete(licSpec);
		return FLX_ACT_FALSE;
	}
	frCount = flxActCommonLicSpcGetNumberProducts(licSpec);

	/*
		Iterate over the fulfillments, listing those that the filters allow.
	*/
	for (frIndex = 0; frIndex < frCount; frIndex++)
	{
		prodSpec = flxActCommonLicSpcGet(licSpec, frIndex);
		if (sIsViewFilterPass(prodSpec, &filter))
		{
			sViewFulfillmentShort(prodSpec, FLX_ACT_FALSE /* No headings */);
			deleteCount++;
		}
	}

	if (frCount == 0)
		flxCtuPrintMessage("\nNo fulfillment records in trusted storage.\n");
	else if (deleteCount == 0)
		flxCtuPrintMessage("No records to delete.\n");
	else
	{
		if (
			flxCtuConfirm("%u record%s to delete. Confirm?\n", 
						  deleteCount, 
						  (deleteCount == 1)? "" : "s"))
		{
			/* Iterate over the fulfillments, deleting those that the filters pass.	*/
			for (frIndex = 0, bResult = FLX_ACT_TRUE; frIndex < frCount; frIndex++)
			{				
				prodSpec = flxActCommonLicSpcGet(licSpec, frIndex);
				if (sIsViewFilterPass(prodSpec, &filter))
				{
					flxCtuPrintMessage("Deleting fulfillment %s\n", 
									   flxActCommonProdLicSpcFulfillmentIdGet(prodSpec));
					if ( !flxActCommonLicSpcProductDelete(licSpec, prodSpec))
					{
						flxCtuContextPrintLastApiFlexError(pContext);
						bResult = FLX_ACT_FALSE;
						break;
					}
				}
			}
		}
	}

	flxActCommonLicSpcDelete(licSpec);
	return bResult;
}

/***************************************************************************************************
*	flxCtuViewShowUsage
*
* Show the command line usage for the -view command.
***************************************************************************************************/
void flxCtuViewShowUsage(flxCtuContext* pContext)
{
	flxCtuPrintOutput(
		"\n"
		"view command: view the fulfillment records in trusted storage, including those\n"
		"              created by means other than composite transactions.\n"
		"\n"
		"    -view|-v [<fulfillmentid>] [<filters>] [format=<formatType>]\n"	
		"\n"
		"  Filters:\n"
		"    fulfillmentId        The record with this id.\n"
		"    product=<productId>  Only records with this ProductId.\n"
		"    feature=<string>     Only records with <string> in the feature line.\n"
		"    trusted=yes|no       Only trusted|untrusted records.\n"
		"    enabled=yes|no       Only enabled|disabled records.\n"
		"\n"
		"  Format types:\n"
		"    default   U D FulfillmentType Expiration Count FulfillmentId ProductId\n"
		"              where U is replaced by . if trusted,  D by . if enabled.\n"
		"    long      Multi-line with format <name>=<value>\n"
		"\n"
		"  Example - all records in default format:\n"
		"      -v \n"
		"\n"
		"  Example - records for \"MyProduct\" with feature \"ADD\":\n"
		"      -v product=MyProduct feature=ADD\n"
		"\n"
		"  Example - record with fulfillment id \"FID_123\", multi-line:\n"
		"      -v FID-123 format=long\n"
		"\n"
		"  Example - disabled records:\n"
		"      -v enabled=no\n"
		);
}

/***************************************************************************************************
*	flxCtuViewFulfillments
*
* Process a view command - view fulfillment records.
* For command line usage see above.
***************************************************************************************************/
FlxActBool flxCtuViewFulfillments(flxCtuContext* pContext, const flxCtuCommand* pCommand)
{
	FlxActBool			bResult			= FLX_ACT_FALSE;
	FlxActLicSpc		licSpec			= 0;
	FlxActProdLicSpc	prodSpec		= 0;
	uint32_t			frCount			= 0;
	uint32_t			viewCount		= 0;
	uint32_t			frIndex			= 0;
	FlxActBool			bPrintHeading	= FLX_ACT_TRUE;
	FlxCtuViewType		viewType		= sGetViewType(pCommand);
	FlxCtuViewFilter	filter;

	/*
		Set the filtering choices from the command value and attributes.
	*/
	sSetViewFilter(pCommand, &filter);

	/*
		Get the fulfillment records.
		The FlxActProdSpc object prodSpec holds details of one fulfillment record.
		The FlxActProdLicSpc object licSpec is a container of prodSpecs.
	*/
	if ( !flxActCommonLicSpcCreate(pContext->hApi, &licSpec) )
	{
		flxCtuContextPrintLastApiFlexError(pContext);
		return FLX_ACT_FALSE;
	}
	if ( !flxActCommonLicSpcPopulateAllFromTS(licSpec) )
	{
		flxCtuContextPrintLastApiFlexError(pContext);
		flxActCommonLicSpcDelete(licSpec);
		return FLX_ACT_FALSE;
	}
	frCount = flxActCommonLicSpcGetNumberProducts(licSpec);

	/*
		Iterate over the fulfillments, viewing those that the filters allow.
	*/
	for (frIndex = 0; frIndex < frCount; frIndex++)
	{
		prodSpec = flxActCommonLicSpcGet(licSpec, frIndex);
		if (sIsViewFilterPass(prodSpec, &filter))
		{
			if (viewType == flxCtuViewTypeLong)
				sViewFulfillmentLong(prodSpec, sIsViewCounts(pContext, pCommand));
			else
				sViewFulfillmentShort(prodSpec, bPrintHeading);

			bPrintHeading = FLX_ACT_FALSE;	/* First time only */

			viewCount++;
		}
	}

	if (frCount == 0)
		flxCtuPrintMessage("\nNo fulfillment records in trusted storage.\n");
	else
		flxCtuPrintMessage("\nViewed %d of %d fulfillment records.\n", viewCount, frCount);

	flxActCommonLicSpcDelete(licSpec);
	return FLX_ACT_TRUE;
}

const char* sGetFulfillmentTrustFlagsString(FlxActProdLicSpc prodSpec)
{
	uint32_t trustFlags = flxActCommonProdLicSpcTrustFlagsGet(prodSpec);
	static char szTrustString[32];

	if (trustFlags == FLAGS_FULLY_TRUSTED)
	{
		sprintf(szTrustString, "0x%X Fully Trusted", trustFlags);
	}
	else
	{
		sprintf(szTrustString, 
			"0x%X **BROKEN** %s%s%s", 
			trustFlags,
			((trustFlags & FLAGS_TRUST_RESTORE) == 0)? "Restore " : "",
			((trustFlags & FLAGS_TRUST_HOST)    == 0)? "Host "    : "",
			((trustFlags & FLAGS_TRUST_TIME)    == 0)? "Time "    : ""
			);
	}
	return szTrustString;
}

const char* sGetFulfillmentTypeString(FlxActProdLicSpc prodSpec)
{
	static const char *szType[] = {	"0 Unknown", 
									"1 Trial", 
									"2 Shortcode",  
									"3 Emergency", 
									"4 Served Activation", 
									"5 Served Overdraft Activation",
									"6 Publisher Activation", 
									"7 Publisher Overdraft Activation"};

	FlxFulfillmentType fulfillmentType = flxActCommonProdLicSpcFulfillmentTypeGet(prodSpec);
	if ( fulfillmentType >= (sizeof(szType) / sizeof(szType[0])) )
		fulfillmentType = FULFILLMENT_TYPE_UNKNOWN;
	return szType[fulfillmentType];
}

uint32_t sGetNonOverdraftCountTotal(FlxActProdLicSpc prodSpec)
{
	return	flxActCommonProdLicSpcCountGet(prodSpec, CONCURRENT)
		  +	flxActCommonProdLicSpcCountGet(prodSpec, ACTIVATABLE)
		  +	flxActCommonProdLicSpcCountGet(prodSpec, HYBRID);
}

FlxCtuViewType sGetViewType(const flxCtuCommand* pCommand)
{
	FlxCtuViewType viewType = flxCtuViewTypeShort;
	if (flxCtuCommandHasAttribute(pCommand, CMD_ATTR_FORMAT))
	{
		if (flxCtuCommandAttributeIs(pCommand, CMD_ATTR_FORMAT, "long"))
			viewType = flxCtuViewTypeLong;
	}
	return viewType;
}

FlxActBool sIsViewCounts(const flxCtuContext* pContext, const flxCtuCommand* pCommand)
{
	/*
		Default is true for server, false for application.
	*/
	return flxCtuCommandAttributeIsYes(pCommand, "counts", pContext->bIsServer);
}

FlxActBool sIsViewFilterPass(FlxActProdLicSpc prodSpec, const FlxCtuViewFilter* pFilter)
{
	FlxActBool bIsPass = FLX_ACT_FALSE;

	if (pFilter->bViewAllIds || sHasThisFulfillmentId(prodSpec, pFilter->pszFidFilter))
	{
		if (pFilter->bViewAllProducts || sHasThisProductId(prodSpec, pFilter->pszPidFilter))
		{
			if (   pFilter->bViewAllFeatures 
				|| sHasThisFeatureToken(prodSpec, pFilter->pszFeatureFilter))
			{
				uint32_t trustFlags = flxActCommonProdLicSpcTrustFlagsGet(prodSpec);
				if (   ((trustFlags == FLAGS_FULLY_TRUSTED) || pFilter->bViewUntrusted)
					|| ((trustFlags != FLAGS_FULLY_TRUSTED) || pFilter->bViewTrusted) )
				{
					FlxActBool bIsDisabled = flxActCommonProdLicSpcIsDisabled(prodSpec);
					if (   (!bIsDisabled || pFilter->bViewEnabled)
						|| ( bIsDisabled || pFilter->bViewDisabled) )
					{
						bIsPass = FLX_ACT_TRUE;
					}
				}
			}
		}
	}
	return bIsPass;
}

/***************************************************************************************************
*	sHasThisFeatureToken 
*
* Returns true if the token appears anywhere in the feature line. 
***************************************************************************************************/
FlxActBool sHasThisFeatureToken(FlxActProdLicSpc prodSpec, const char* pszFeatureToken)
{
	const char* pszFeatureLine = flxActCommonProdLicSpcFeatureLineGet(prodSpec);

	if (pszFeatureLine == NULL)
		return FLX_ACT_FALSE;

	return (strstr(pszFeatureLine, pszFeatureToken) == NULL)? FLX_ACT_FALSE : FLX_ACT_TRUE;
}

FlxActBool sHasThisFulfillmentId(FlxActProdLicSpc prodSpec, const char* pszFulfillmentId)
{
	return (0 == strcmp(pszFulfillmentId, flxActCommonProdLicSpcFulfillmentIdGet(prodSpec)));
}

FlxActBool sHasThisProductId(FlxActProdLicSpc prodSpec, const char* pszProductId)
{
	const char* pszFrProductId = flxActCommonProdLicSpcProductIdGet(prodSpec);

	if (pszFrProductId == NULL)
		return FLX_ACT_FALSE;

	return (0 == strcmp(pszProductId, pszFrProductId));
}

void sSetViewFilter(const flxCtuCommand* pCommand, FlxCtuViewFilter* pFilter)
{
	pFilter->bViewAllIds		= !flxCtuCommandHasValue(pCommand);
	pFilter->bViewAllProducts	= !flxCtuCommandHasAttribute(pCommand, CMD_ATTR_PID);
	pFilter->bViewAllFeatures	= !flxCtuCommandHasAttribute(pCommand, CMD_ATTR_FEATURE);
	pFilter->bViewTrusted		= !flxCtuCommandAttributeIs(pCommand, CMD_ATTR_TRUSTED, "no");
	pFilter->bViewUntrusted		= !flxCtuCommandAttributeIs(pCommand, CMD_ATTR_TRUSTED, "yes");
	pFilter->bViewEnabled		= !flxCtuCommandAttributeIs(pCommand, CMD_ATTR_ENABLED, "no");
	pFilter->bViewDisabled		= !flxCtuCommandAttributeIs(pCommand, CMD_ATTR_ENABLED, "yes");

	pFilter->pszFidFilter		= pFilter->bViewAllIds? "" : flxCtuCommandGetValue(pCommand);
	pFilter->pszPidFilter		= pFilter->bViewAllProducts?
								  "" : flxCtuCommandGetAttribute(pCommand, CMD_ATTR_PID);
	pFilter->pszFeatureFilter	= pFilter->bViewAllFeatures?
								  "" : flxCtuCommandGetAttribute(pCommand, CMD_ATTR_FEATURE);
}

void sViewFulfillmentCounts(FlxActProdLicSpc prodSpec)
{
	flxCtuPrintOutput("\n");
	sViewNameAndUint32("Concurrent", flxActCommonProdLicSpcCountGet(prodSpec, CONCURRENT));
	sViewNameAndUint32("Activatable", flxActCommonProdLicSpcCountGet(prodSpec, ACTIVATABLE));
	sViewNameAndUint32("Hybrid", flxActCommonProdLicSpcCountGet(prodSpec, HYBRID));
	sViewNameAndUint32("Concurrent overdraft", flxActCommonProdLicSpcCountGet(prodSpec, 
																			   CONCURRENT_OD));
	sViewNameAndUint32("Activatable overdraft", flxActCommonProdLicSpcCountGet(prodSpec, 
																			    ACTIVATABLE_OD));
	sViewNameAndUint32("Hybrid overdraft", flxActCommonProdLicSpcCountGet(prodSpec, HYBRID_OD));
	sViewNameAndUint32("Repair", flxActCommonProdLicSpcCountGet(prodSpec, REPAIRS));
}

static void sViewFulfillmentLong(FlxActProdLicSpc prodSpec, FlxActBool bViewCounts)
{
	flxCtuPrintOutput(
		"-------------------------------------------------------------------------------\n");
	sViewNameAndValue("Fulfillment Id", flxActCommonProdLicSpcFulfillmentIdGet(prodSpec));
	sViewNameAndValue("Fulfillment Type", sGetFulfillmentTypeString(prodSpec));
	sViewNameAndValue("Trust Flags", sGetFulfillmentTrustFlagsString(prodSpec));
	sViewNameAndValue("Status", flxActCommonProdLicSpcIsDisabled(prodSpec)? "Disabled" : "Enabled");
	sViewNameAndValue("Entitlement Id", flxActCommonProdLicSpcEntitlementIdGet(prodSpec));
	sViewNameAndValue("Product Id", flxActCommonProdLicSpcProductIdGet(prodSpec));
	sViewNameAndValue("Expiration Date", flxActCommonProdLicSpcExpDateGet(prodSpec));
	sViewNameAndValue("Server chain", flxActCommonProdLicSpcActServerChainGet(prodSpec));
	sViewNameAndFeatures("Feature Line(s)", prodSpec);

	if (bViewCounts)
		sViewFulfillmentCounts(prodSpec);
}

void sViewFulfillmentShort(FlxActProdLicSpc prodSpec, FlxActBool bPrintHeading)
{
	FlxActBool bIsTrusted  = (flxActCommonProdLicSpcTrustFlagsGet(prodSpec) == FLAGS_FULLY_TRUSTED);
	FlxActBool bIsDisabled = flxActCommonProdLicSpcIsDisabled(prodSpec);

	FlxFulfillmentType fulfillmentType = flxActCommonProdLicSpcFulfillmentTypeGet(prodSpec);

	const char* pszFulfillmentId = flxActCommonProdLicSpcFulfillmentIdGet(prodSpec);
	const char* pszProductId	 = flxActCommonProdLicSpcProductIdGet(prodSpec);
	const char* pszExpiry		 = flxActCommonProdLicSpcExpDateGet(prodSpec);
	uint32_t nonOverdraftCount   = sGetNonOverdraftCountTotal(prodSpec);

	if (pszFulfillmentId == NULL)
		pszFulfillmentId = "<NoFid>";

	if (pszProductId == NULL)
		pszProductId = "<NoPid>";

	if (pszExpiry == NULL)
		pszExpiry = "<NoExp>";
	
	if (bPrintHeading)
	{
		flxCtuPrintMessage(
			"\n"
			"U if untrusted\n"
			". D if disabled\n"
			". . fulfillmentType\n" 
			". . . Expiry   Count FulfillmentId                   ProductId\n"
			"\n");
	}

	flxCtuPrintOutput("%c %c %u %-11.11s %2u %-31s %s\n",
					  bIsTrusted?  '.' : 'U',
					  bIsDisabled? 'D' : '.',
					  fulfillmentType,
					  pszExpiry,
					  nonOverdraftCount,
					  pszFulfillmentId,
					  pszProductId
				     );
}

void sViewNameAndFeatures(const char* pszName, FlxActProdLicSpc prodSpec)
{
	const char* pszFeatureLine = flxActCommonProdLicSpcFeatureLineGet(prodSpec);

	sViewNameAndValue(pszName, "");
	if (pszFeatureLine != NULL)
	{
		flxCtuPrintOutput(pszFeatureLine);
	}
	flxCtuPrintOutput("\n");
}

void sViewNameAndValue(const char* pszName, const char* pszValue)
{
	flxCtuPrintOutput("%22.22s : %s\n", 
					  pszName,
					  (pszValue == NULL)? "" : pszValue
					 );
}

void sViewNameAndUint32(const char* pszName, uint32_t value)
{
	char pszValue[sizeof(value) * 3 + 1];
	sprintf(pszValue, "%u", value);
	sViewNameAndValue(pszName, pszValue);
}
